@@ -428,13 +428,10 @@ static bool si_update_draw_info_state(struct si_context *sctx,
 		bool ia_switch_on_eop = wd_switch_on_eop;
 		unsigned primgroup_size = 64;
 
-		/* Hawaii hangs if instancing is enabled and each instance
-		 * is smaller than a prim group and WD_SWITCH_ON_EOP is 0.
+		/* Hawaii hangs if instancing is enabled and WD_SWITCH_ON_EOP is 0.
 		 * We don't know that for indirect drawing, so treat it as
 		 * always problematic. */
-		if (sctx->b.family == CHIP_HAWAII &&
-		    ((info->instance_count > 1 &&
-		      u_prims_for_vertices(info->mode, info->count) < primgroup_size))) {
+		if (sctx->b.family == CHIP_HAWAII && info->instance_count > 1) {
 			wd_switch_on_eop = true;
 			ia_switch_on_eop = true;
 		}
@@ -984,7 +981,7 @@ void si_draw_vbo(struct pipe_context *ctx, const struct pipe_draw_info *info)
 
 	/* Check flush flags. */
 	if (sctx->b.flags)
-		sctx->atoms.cache_flush->dirty = true;
+		sctx->atoms.s.cache_flush->dirty = true;
 
 	si_need_cs_space(sctx, 0, TRUE);
 
