@@ -310,7 +310,7 @@ const struct surface_format_info surface_formats[] = {
 #undef Y
 
 uint32_t
-brw_format_for_mesa_format(gl_format mesa_format)
+brw_format_for_mesa_format(mesa_format mesa_format)
 {
    /* This table is ordered according to the enum ordering in formats.h.  We do
     * expect that enum to be extended without our explicit initialization
@@ -319,57 +319,58 @@ brw_format_for_mesa_format(gl_format mesa_format)
     */
    static const uint32_t table[MESA_FORMAT_COUNT] =
    {
-      [MESA_FORMAT_RGBA8888] = 0,
-      [MESA_FORMAT_RGBA8888_REV] = BRW_SURFACEFORMAT_R8G8B8A8_UNORM,
-      [MESA_FORMAT_ARGB8888] = BRW_SURFACEFORMAT_B8G8R8A8_UNORM,
-      [MESA_FORMAT_ARGB8888_REV] = 0,
-      [MESA_FORMAT_RGBX8888] = 0,
-      [MESA_FORMAT_RGBX8888_REV] = BRW_SURFACEFORMAT_R8G8B8X8_UNORM,
-      [MESA_FORMAT_XRGB8888] = BRW_SURFACEFORMAT_B8G8R8X8_UNORM,
-      [MESA_FORMAT_XRGB8888_REV] = 0,
-      [MESA_FORMAT_RGB888] = 0,
-      [MESA_FORMAT_BGR888] = BRW_SURFACEFORMAT_R8G8B8_UNORM,
-      [MESA_FORMAT_RGB565] = BRW_SURFACEFORMAT_B5G6R5_UNORM,
-      [MESA_FORMAT_RGB565_REV] = 0,
-      [MESA_FORMAT_ARGB4444] = BRW_SURFACEFORMAT_B4G4R4A4_UNORM,
-      [MESA_FORMAT_ARGB4444_REV] = 0,
-      [MESA_FORMAT_RGBA5551] = 0,
-      [MESA_FORMAT_ARGB1555] = BRW_SURFACEFORMAT_B5G5R5A1_UNORM,
-      [MESA_FORMAT_ARGB1555_REV] = 0,
-      [MESA_FORMAT_AL44] = 0,
-      [MESA_FORMAT_AL88] = BRW_SURFACEFORMAT_L8A8_UNORM,
-      [MESA_FORMAT_AL88_REV] = 0,
-      [MESA_FORMAT_AL1616] = BRW_SURFACEFORMAT_L16A16_UNORM,
-      [MESA_FORMAT_AL1616_REV] = 0,
-      [MESA_FORMAT_RGB332] = 0,
-      [MESA_FORMAT_A8] = BRW_SURFACEFORMAT_A8_UNORM,
-      [MESA_FORMAT_A16] = BRW_SURFACEFORMAT_A16_UNORM,
-      [MESA_FORMAT_L8] = BRW_SURFACEFORMAT_L8_UNORM,
-      [MESA_FORMAT_L16] = BRW_SURFACEFORMAT_L16_UNORM,
-      [MESA_FORMAT_I8] = BRW_SURFACEFORMAT_I8_UNORM,
-      [MESA_FORMAT_I16] = BRW_SURFACEFORMAT_I16_UNORM,
+      [MESA_FORMAT_A8B8G8R8_UNORM] = 0,
+      [MESA_FORMAT_R8G8B8A8_UNORM] = BRW_SURFACEFORMAT_R8G8B8A8_UNORM,
+      [MESA_FORMAT_B8G8R8A8_UNORM] = BRW_SURFACEFORMAT_B8G8R8A8_UNORM,
+      [MESA_FORMAT_A8R8G8B8_UNORM] = 0,
+      [MESA_FORMAT_X8B8G8R8_UNORM] = 0,
+      [MESA_FORMAT_R8G8B8X8_UNORM] = BRW_SURFACEFORMAT_R8G8B8X8_UNORM,
+      [MESA_FORMAT_B8G8R8X8_UNORM] = BRW_SURFACEFORMAT_B8G8R8X8_UNORM,
+      [MESA_FORMAT_X8R8G8B8_UNORM] = 0,
+      [MESA_FORMAT_BGR_UNORM8] = 0,
+      [MESA_FORMAT_RGB_UNORM8] = BRW_SURFACEFORMAT_R8G8B8_UNORM,
+      [MESA_FORMAT_B5G6R5_UNORM] = BRW_SURFACEFORMAT_B5G6R5_UNORM,
+      [MESA_FORMAT_R5G6B5_UNORM] = 0,
+      [MESA_FORMAT_B4G4R4A4_UNORM] = BRW_SURFACEFORMAT_B4G4R4A4_UNORM,
+      [MESA_FORMAT_A4R4G4B4_UNORM] = 0,
+      [MESA_FORMAT_A1B5G5R5_UNORM] = 0,
+      [MESA_FORMAT_B5G5R5A1_UNORM] = BRW_SURFACEFORMAT_B5G5R5A1_UNORM,
+      [MESA_FORMAT_A1R5G5B5_UNORM] = 0,
+      [MESA_FORMAT_L4A4_UNORM] = 0,
+      [MESA_FORMAT_L8A8_UNORM] = BRW_SURFACEFORMAT_L8A8_UNORM,
+      [MESA_FORMAT_A8L8_UNORM] = 0,
+      [MESA_FORMAT_L16A16_UNORM] = BRW_SURFACEFORMAT_L16A16_UNORM,
+      [MESA_FORMAT_A16L16_UNORM] = 0,
+      [MESA_FORMAT_B2G3R3_UNORM] = 0,
+      [MESA_FORMAT_A_UNORM8] = BRW_SURFACEFORMAT_A8_UNORM,
+      [MESA_FORMAT_A_UNORM16] = BRW_SURFACEFORMAT_A16_UNORM,
+      [MESA_FORMAT_L_UNORM8] = BRW_SURFACEFORMAT_L8_UNORM,
+      [MESA_FORMAT_L_UNORM16] = BRW_SURFACEFORMAT_L16_UNORM,
+      [MESA_FORMAT_I_UNORM8] = BRW_SURFACEFORMAT_I8_UNORM,
+      [MESA_FORMAT_I_UNORM16] = BRW_SURFACEFORMAT_I16_UNORM,
       [MESA_FORMAT_YCBCR_REV] = BRW_SURFACEFORMAT_YCRCB_NORMAL,
       [MESA_FORMAT_YCBCR] = BRW_SURFACEFORMAT_YCRCB_SWAPUVY,
-      [MESA_FORMAT_R8] = BRW_SURFACEFORMAT_R8_UNORM,
-      [MESA_FORMAT_GR88] = BRW_SURFACEFORMAT_R8G8_UNORM,
-      [MESA_FORMAT_RG88] = 0,
-      [MESA_FORMAT_R16] = BRW_SURFACEFORMAT_R16_UNORM,
-      [MESA_FORMAT_GR1616] = BRW_SURFACEFORMAT_R16G16_UNORM,
-      [MESA_FORMAT_RG1616] = 0,
-      [MESA_FORMAT_ARGB2101010] = BRW_SURFACEFORMAT_B10G10R10A2_UNORM,
-      [MESA_FORMAT_Z24_S8] = 0,
-      [MESA_FORMAT_S8_Z24] = 0,
-      [MESA_FORMAT_Z16] = 0,
-      [MESA_FORMAT_X8_Z24] = 0,
-      [MESA_FORMAT_Z24_X8] = 0,
-      [MESA_FORMAT_Z32] = 0,
-      [MESA_FORMAT_S8] = 0,
+      [MESA_FORMAT_R_UNORM8] = BRW_SURFACEFORMAT_R8_UNORM,
+      [MESA_FORMAT_R8G8_UNORM] = BRW_SURFACEFORMAT_R8G8_UNORM,
+      [MESA_FORMAT_G8R8_UNORM] = 0,
+      [MESA_FORMAT_R_UNORM16] = BRW_SURFACEFORMAT_R16_UNORM,
+      [MESA_FORMAT_R16G16_UNORM] = BRW_SURFACEFORMAT_R16G16_UNORM,
+      [MESA_FORMAT_G16R16_UNORM] = 0,
+      [MESA_FORMAT_B10G10R10A2_UNORM] = BRW_SURFACEFORMAT_B10G10R10A2_UNORM,
+      [MESA_FORMAT_S8_UINT_Z24_UNORM] = 0,
+      [MESA_FORMAT_Z24_UNORM_S8_UINT] = 0,
+      [MESA_FORMAT_Z_UNORM16] = 0,
+      [MESA_FORMAT_Z24_UNORM_X8_UINT] = 0,
+      [MESA_FORMAT_X8_UINT_Z24_UNORM] = 0,
+      [MESA_FORMAT_Z_UNORM32] = 0,
+      [MESA_FORMAT_S_UINT8] = BRW_SURFACEFORMAT_R8_UINT,
 
-      [MESA_FORMAT_SRGB8] = 0,
-      [MESA_FORMAT_SRGBA8] = 0,
-      [MESA_FORMAT_SARGB8] = BRW_SURFACEFORMAT_B8G8R8A8_UNORM_SRGB,
-      [MESA_FORMAT_SL8] = BRW_SURFACEFORMAT_L8_UNORM_SRGB,
-      [MESA_FORMAT_SLA8] = BRW_SURFACEFORMAT_L8A8_UNORM_SRGB,
+      [MESA_FORMAT_BGR_SRGB8] = 0,
+      [MESA_FORMAT_A8B8G8R8_SRGB] = 0,
+      [MESA_FORMAT_B8G8R8A8_SRGB] = BRW_SURFACEFORMAT_B8G8R8A8_UNORM_SRGB,
+      [MESA_FORMAT_R8G8B8A8_SRGB] = BRW_SURFACEFORMAT_R8G8B8A8_UNORM_SRGB,
+      [MESA_FORMAT_L_SRGB8] = BRW_SURFACEFORMAT_L8_UNORM_SRGB,
+      [MESA_FORMAT_L8A8_SRGB] = BRW_SURFACEFORMAT_L8A8_UNORM_SRGB,
       [MESA_FORMAT_SRGB_DXT1] = BRW_SURFACEFORMAT_DXT1_RGB_SRGB,
       [MESA_FORMAT_SRGBA_DXT1] = BRW_SURFACEFORMAT_BC1_UNORM_SRGB,
       [MESA_FORMAT_SRGBA_DXT3] = BRW_SURFACEFORMAT_BC2_UNORM_SRGB,
@@ -385,60 +386,60 @@ brw_format_for_mesa_format(gl_format mesa_format)
       [MESA_FORMAT_RGBA_FLOAT32] = BRW_SURFACEFORMAT_R32G32B32A32_FLOAT,
       [MESA_FORMAT_RGBA_FLOAT16] = BRW_SURFACEFORMAT_R16G16B16A16_FLOAT,
       [MESA_FORMAT_RGB_FLOAT32] = BRW_SURFACEFORMAT_R32G32B32_FLOAT,
-      [MESA_FORMAT_RGB_FLOAT16] = BRW_SURFACEFORMAT_R16G16B16_FLOAT,
-      [MESA_FORMAT_ALPHA_FLOAT32] = BRW_SURFACEFORMAT_A32_FLOAT,
-      [MESA_FORMAT_ALPHA_FLOAT16] = BRW_SURFACEFORMAT_A16_FLOAT,
-      [MESA_FORMAT_LUMINANCE_FLOAT32] = BRW_SURFACEFORMAT_L32_FLOAT,
-      [MESA_FORMAT_LUMINANCE_FLOAT16] = BRW_SURFACEFORMAT_L16_FLOAT,
-      [MESA_FORMAT_LUMINANCE_ALPHA_FLOAT32] = BRW_SURFACEFORMAT_L32A32_FLOAT,
-      [MESA_FORMAT_LUMINANCE_ALPHA_FLOAT16] = BRW_SURFACEFORMAT_L16A16_FLOAT,
-      [MESA_FORMAT_INTENSITY_FLOAT32] = BRW_SURFACEFORMAT_I32_FLOAT,
-      [MESA_FORMAT_INTENSITY_FLOAT16] = BRW_SURFACEFORMAT_I16_FLOAT,
+      [MESA_FORMAT_RGB_FLOAT16] = 0,
+      [MESA_FORMAT_A_FLOAT32] = BRW_SURFACEFORMAT_A32_FLOAT,
+      [MESA_FORMAT_A_FLOAT16] = BRW_SURFACEFORMAT_A16_FLOAT,
+      [MESA_FORMAT_L_FLOAT32] = BRW_SURFACEFORMAT_L32_FLOAT,
+      [MESA_FORMAT_L_FLOAT16] = BRW_SURFACEFORMAT_L16_FLOAT,
+      [MESA_FORMAT_LA_FLOAT32] = BRW_SURFACEFORMAT_L32A32_FLOAT,
+      [MESA_FORMAT_LA_FLOAT16] = BRW_SURFACEFORMAT_L16A16_FLOAT,
+      [MESA_FORMAT_I_FLOAT32] = BRW_SURFACEFORMAT_I32_FLOAT,
+      [MESA_FORMAT_I_FLOAT16] = BRW_SURFACEFORMAT_I16_FLOAT,
       [MESA_FORMAT_R_FLOAT32] = BRW_SURFACEFORMAT_R32_FLOAT,
       [MESA_FORMAT_R_FLOAT16] = BRW_SURFACEFORMAT_R16_FLOAT,
       [MESA_FORMAT_RG_FLOAT32] = BRW_SURFACEFORMAT_R32G32_FLOAT,
       [MESA_FORMAT_RG_FLOAT16] = BRW_SURFACEFORMAT_R16G16_FLOAT,
 
-      [MESA_FORMAT_ALPHA_UINT8] = 0,
-      [MESA_FORMAT_ALPHA_UINT16] = 0,
-      [MESA_FORMAT_ALPHA_UINT32] = 0,
-      [MESA_FORMAT_ALPHA_INT8] = 0,
-      [MESA_FORMAT_ALPHA_INT16] = 0,
-      [MESA_FORMAT_ALPHA_INT32] = 0,
+      [MESA_FORMAT_A_UINT8] = 0,
+      [MESA_FORMAT_A_UINT16] = 0,
+      [MESA_FORMAT_A_UINT32] = 0,
+      [MESA_FORMAT_A_SINT8] = 0,
+      [MESA_FORMAT_A_SINT16] = 0,
+      [MESA_FORMAT_A_SINT32] = 0,
 
-      [MESA_FORMAT_INTENSITY_UINT8] = 0,
-      [MESA_FORMAT_INTENSITY_UINT16] = 0,
-      [MESA_FORMAT_INTENSITY_UINT32] = 0,
-      [MESA_FORMAT_INTENSITY_INT8] = 0,
-      [MESA_FORMAT_INTENSITY_INT16] = 0,
-      [MESA_FORMAT_INTENSITY_INT32] = 0,
+      [MESA_FORMAT_I_UINT8] = 0,
+      [MESA_FORMAT_I_UINT16] = 0,
+      [MESA_FORMAT_I_UINT32] = 0,
+      [MESA_FORMAT_I_SINT8] = 0,
+      [MESA_FORMAT_I_SINT16] = 0,
+      [MESA_FORMAT_I_SINT32] = 0,
 
-      [MESA_FORMAT_LUMINANCE_UINT8] = 0,
-      [MESA_FORMAT_LUMINANCE_UINT16] = 0,
-      [MESA_FORMAT_LUMINANCE_UINT32] = 0,
-      [MESA_FORMAT_LUMINANCE_INT8] = 0,
-      [MESA_FORMAT_LUMINANCE_INT16] = 0,
-      [MESA_FORMAT_LUMINANCE_INT32] = 0,
+      [MESA_FORMAT_L_UINT8] = 0,
+      [MESA_FORMAT_L_UINT16] = 0,
+      [MESA_FORMAT_L_UINT32] = 0,
+      [MESA_FORMAT_L_SINT8] = 0,
+      [MESA_FORMAT_L_SINT16] = 0,
+      [MESA_FORMAT_L_SINT32] = 0,
 
-      [MESA_FORMAT_LUMINANCE_ALPHA_UINT8] = 0,
-      [MESA_FORMAT_LUMINANCE_ALPHA_UINT16] = 0,
-      [MESA_FORMAT_LUMINANCE_ALPHA_UINT32] = 0,
-      [MESA_FORMAT_LUMINANCE_ALPHA_INT8] = 0,
-      [MESA_FORMAT_LUMINANCE_ALPHA_INT16] = 0,
-      [MESA_FORMAT_LUMINANCE_ALPHA_INT32] = 0,
+      [MESA_FORMAT_LA_UINT8] = 0,
+      [MESA_FORMAT_LA_UINT16] = 0,
+      [MESA_FORMAT_LA_UINT32] = 0,
+      [MESA_FORMAT_LA_SINT8] = 0,
+      [MESA_FORMAT_LA_SINT16] = 0,
+      [MESA_FORMAT_LA_SINT32] = 0,
 
-      [MESA_FORMAT_R_INT8] = BRW_SURFACEFORMAT_R8_SINT,
-      [MESA_FORMAT_RG_INT8] = BRW_SURFACEFORMAT_R8G8_SINT,
-      [MESA_FORMAT_RGB_INT8] = BRW_SURFACEFORMAT_R8G8B8_SINT,
-      [MESA_FORMAT_RGBA_INT8] = BRW_SURFACEFORMAT_R8G8B8A8_SINT,
-      [MESA_FORMAT_R_INT16] = BRW_SURFACEFORMAT_R16_SINT,
-      [MESA_FORMAT_RG_INT16] = BRW_SURFACEFORMAT_R16G16_SINT,
-      [MESA_FORMAT_RGB_INT16] = BRW_SURFACEFORMAT_R16G16B16_SINT,
-      [MESA_FORMAT_RGBA_INT16] = BRW_SURFACEFORMAT_R16G16B16A16_SINT,
-      [MESA_FORMAT_R_INT32] = BRW_SURFACEFORMAT_R32_SINT,
-      [MESA_FORMAT_RG_INT32] = BRW_SURFACEFORMAT_R32G32_SINT,
-      [MESA_FORMAT_RGB_INT32] = BRW_SURFACEFORMAT_R32G32B32_SINT,
-      [MESA_FORMAT_RGBA_INT32] = BRW_SURFACEFORMAT_R32G32B32A32_SINT,
+      [MESA_FORMAT_R_SINT8] = BRW_SURFACEFORMAT_R8_SINT,
+      [MESA_FORMAT_RG_SINT8] = BRW_SURFACEFORMAT_R8G8_SINT,
+      [MESA_FORMAT_RGB_SINT8] = BRW_SURFACEFORMAT_R8G8B8_SINT,
+      [MESA_FORMAT_RGBA_SINT8] = BRW_SURFACEFORMAT_R8G8B8A8_SINT,
+      [MESA_FORMAT_R_SINT16] = BRW_SURFACEFORMAT_R16_SINT,
+      [MESA_FORMAT_RG_SINT16] = BRW_SURFACEFORMAT_R16G16_SINT,
+      [MESA_FORMAT_RGB_SINT16] = BRW_SURFACEFORMAT_R16G16B16_SINT,
+      [MESA_FORMAT_RGBA_SINT16] = BRW_SURFACEFORMAT_R16G16B16A16_SINT,
+      [MESA_FORMAT_R_SINT32] = BRW_SURFACEFORMAT_R32_SINT,
+      [MESA_FORMAT_RG_SINT32] = BRW_SURFACEFORMAT_R32G32_SINT,
+      [MESA_FORMAT_RGB_SINT32] = BRW_SURFACEFORMAT_R32G32B32_SINT,
+      [MESA_FORMAT_RGBA_SINT32] = BRW_SURFACEFORMAT_R32G32B32A32_SINT,
 
       [MESA_FORMAT_R_UINT8] = BRW_SURFACEFORMAT_R8_UINT,
       [MESA_FORMAT_RG_UINT8] = BRW_SURFACEFORMAT_R8G8_UINT,
@@ -454,26 +455,26 @@ brw_format_for_mesa_format(gl_format mesa_format)
       [MESA_FORMAT_RGBA_UINT32] = BRW_SURFACEFORMAT_R32G32B32A32_UINT,
 
       [MESA_FORMAT_DUDV8] = BRW_SURFACEFORMAT_R8G8_SNORM,
-      [MESA_FORMAT_SIGNED_R8] = BRW_SURFACEFORMAT_R8_SNORM,
-      [MESA_FORMAT_SIGNED_RG88_REV] = BRW_SURFACEFORMAT_R8G8_SNORM,
-      [MESA_FORMAT_SIGNED_RGBX8888] = 0,
-      [MESA_FORMAT_SIGNED_RGBA8888] = 0,
-      [MESA_FORMAT_SIGNED_RGBA8888_REV] = BRW_SURFACEFORMAT_R8G8B8A8_SNORM,
-      [MESA_FORMAT_SIGNED_R16] = BRW_SURFACEFORMAT_R16_SNORM,
-      [MESA_FORMAT_SIGNED_GR1616] = BRW_SURFACEFORMAT_R16G16_SNORM,
-      [MESA_FORMAT_SIGNED_RGB_16] = BRW_SURFACEFORMAT_R16G16B16_SNORM,
-      [MESA_FORMAT_SIGNED_RGBA_16] = BRW_SURFACEFORMAT_R16G16B16A16_SNORM,
-      [MESA_FORMAT_RGBA_16] = BRW_SURFACEFORMAT_R16G16B16A16_UNORM,
+      [MESA_FORMAT_R_SNORM8] = BRW_SURFACEFORMAT_R8_SNORM,
+      [MESA_FORMAT_R8G8_SNORM] = BRW_SURFACEFORMAT_R8G8_SNORM,
+      [MESA_FORMAT_X8B8G8R8_SNORM] = 0,
+      [MESA_FORMAT_A8B8G8R8_SNORM] = 0,
+      [MESA_FORMAT_R8G8B8A8_SNORM] = BRW_SURFACEFORMAT_R8G8B8A8_SNORM,
+      [MESA_FORMAT_R_SNORM16] = BRW_SURFACEFORMAT_R16_SNORM,
+      [MESA_FORMAT_R16G16_SNORM] = BRW_SURFACEFORMAT_R16G16_SNORM,
+      [MESA_FORMAT_RGB_SNORM16] = BRW_SURFACEFORMAT_R16G16B16_SNORM,
+      [MESA_FORMAT_RGBA_SNORM16] = BRW_SURFACEFORMAT_R16G16B16A16_SNORM,
+      [MESA_FORMAT_RGBA_UNORM16] = BRW_SURFACEFORMAT_R16G16B16A16_UNORM,
 
-      [MESA_FORMAT_RED_RGTC1] = BRW_SURFACEFORMAT_BC4_UNORM,
-      [MESA_FORMAT_SIGNED_RED_RGTC1] = BRW_SURFACEFORMAT_BC4_SNORM,
-      [MESA_FORMAT_RG_RGTC2] = BRW_SURFACEFORMAT_BC5_UNORM,
-      [MESA_FORMAT_SIGNED_RG_RGTC2] = BRW_SURFACEFORMAT_BC5_SNORM,
+      [MESA_FORMAT_R_RGTC1_UNORM] = BRW_SURFACEFORMAT_BC4_UNORM,
+      [MESA_FORMAT_R_RGTC1_SNORM] = BRW_SURFACEFORMAT_BC4_SNORM,
+      [MESA_FORMAT_RG_RGTC2_UNORM] = BRW_SURFACEFORMAT_BC5_UNORM,
+      [MESA_FORMAT_RG_RGTC2_SNORM] = BRW_SURFACEFORMAT_BC5_SNORM,
 
-      [MESA_FORMAT_L_LATC1] = 0,
-      [MESA_FORMAT_SIGNED_L_LATC1] = 0,
-      [MESA_FORMAT_LA_LATC2] = 0,
-      [MESA_FORMAT_SIGNED_LA_LATC2] = 0,
+      [MESA_FORMAT_L_LATC1_UNORM] = 0,
+      [MESA_FORMAT_L_LATC1_SNORM] = 0,
+      [MESA_FORMAT_LA_LATC2_UNORM] = 0,
+      [MESA_FORMAT_LA_LATC2_SNORM] = 0,
 
       [MESA_FORMAT_ETC1_RGB8] = BRW_SURFACEFORMAT_ETC1_RGB8,
       [MESA_FORMAT_ETC2_RGB8] = BRW_SURFACEFORMAT_ETC2_RGB8,
@@ -487,39 +488,40 @@ brw_format_for_mesa_format(gl_format mesa_format)
       [MESA_FORMAT_ETC2_RGB8_PUNCHTHROUGH_ALPHA1] = BRW_SURFACEFORMAT_ETC2_RGB8_PTA,
       [MESA_FORMAT_ETC2_SRGB8_PUNCHTHROUGH_ALPHA1] = BRW_SURFACEFORMAT_ETC2_SRGB8_PTA,
 
-      [MESA_FORMAT_SIGNED_A8] = 0,
-      [MESA_FORMAT_SIGNED_L8] = 0,
-      [MESA_FORMAT_SIGNED_AL88] = 0,
-      [MESA_FORMAT_SIGNED_I8] = 0,
-      [MESA_FORMAT_SIGNED_A16] = 0,
-      [MESA_FORMAT_SIGNED_L16] = 0,
-      [MESA_FORMAT_SIGNED_AL1616] = 0,
-      [MESA_FORMAT_SIGNED_I16] = 0,
+      [MESA_FORMAT_A_SNORM8] = 0,
+      [MESA_FORMAT_L_SNORM8] = 0,
+      [MESA_FORMAT_L8A8_SNORM] = 0,
+      [MESA_FORMAT_I_SNORM8] = 0,
+      [MESA_FORMAT_A_SNORM16] = 0,
+      [MESA_FORMAT_L_SNORM16] = 0,
+      [MESA_FORMAT_LA_SNORM16] = 0,
+      [MESA_FORMAT_I_SNORM16] = 0,
 
-      [MESA_FORMAT_RGB9_E5_FLOAT] = BRW_SURFACEFORMAT_R9G9B9E5_SHAREDEXP,
-      [MESA_FORMAT_R11_G11_B10_FLOAT] = BRW_SURFACEFORMAT_R11G11B10_FLOAT,
+      [MESA_FORMAT_R9G9B9E5_FLOAT] = BRW_SURFACEFORMAT_R9G9B9E5_SHAREDEXP,
+      [MESA_FORMAT_R11G11B10_FLOAT] = BRW_SURFACEFORMAT_R11G11B10_FLOAT,
 
-      [MESA_FORMAT_Z32_FLOAT] = 0,
-      [MESA_FORMAT_Z32_FLOAT_X24S8] = 0,
+      [MESA_FORMAT_Z_FLOAT32] = 0,
+      [MESA_FORMAT_Z32_FLOAT_S8X24_UINT] = 0,
 
-      [MESA_FORMAT_ARGB2101010_UINT] = BRW_SURFACEFORMAT_B10G10R10A2_UINT,
-      [MESA_FORMAT_ABGR2101010_UINT] = BRW_SURFACEFORMAT_R10G10B10A2_UINT,
+      [MESA_FORMAT_R10G10B10A2_UNORM] = BRW_SURFACEFORMAT_R10G10B10A2_UNORM,
+      [MESA_FORMAT_B10G10R10A2_UINT] = BRW_SURFACEFORMAT_B10G10R10A2_UINT,
+      [MESA_FORMAT_R10G10B10A2_UINT] = BRW_SURFACEFORMAT_R10G10B10A2_UINT,
 
-      [MESA_FORMAT_XRGB4444_UNORM] = 0,
-      [MESA_FORMAT_XRGB1555_UNORM] = BRW_SURFACEFORMAT_B5G5R5X1_UNORM,
-      [MESA_FORMAT_XBGR8888_SNORM] = 0,
-      [MESA_FORMAT_XBGR8888_SRGB] = 0,
-      [MESA_FORMAT_XBGR8888_UINT] = 0,
-      [MESA_FORMAT_XBGR8888_SINT] = 0,
-      [MESA_FORMAT_XRGB2101010_UNORM] = BRW_SURFACEFORMAT_B10G10R10X2_UNORM,
-      [MESA_FORMAT_XBGR16161616_UNORM] = BRW_SURFACEFORMAT_R16G16B16X16_UNORM,
-      [MESA_FORMAT_XBGR16161616_SNORM] = 0,
-      [MESA_FORMAT_XBGR16161616_FLOAT] = BRW_SURFACEFORMAT_R16G16B16X16_FLOAT,
-      [MESA_FORMAT_XBGR16161616_UINT] = 0,
-      [MESA_FORMAT_XBGR16161616_SINT] = 0,
-      [MESA_FORMAT_XBGR32323232_FLOAT] = BRW_SURFACEFORMAT_R32G32B32X32_FLOAT,
-      [MESA_FORMAT_XBGR32323232_UINT] = 0,
-      [MESA_FORMAT_XBGR32323232_SINT] = 0,
+      [MESA_FORMAT_B4G4R4X4_UNORM] = 0,
+      [MESA_FORMAT_B5G5R5X1_UNORM] = BRW_SURFACEFORMAT_B5G5R5X1_UNORM,
+      [MESA_FORMAT_R8G8B8X8_SNORM] = 0,
+      [MESA_FORMAT_R8G8B8X8_SRGB] = 0,
+      [MESA_FORMAT_RGBX_UINT8] = 0,
+      [MESA_FORMAT_RGBX_SINT8] = 0,
+      [MESA_FORMAT_B10G10R10X2_UNORM] = BRW_SURFACEFORMAT_B10G10R10X2_UNORM,
+      [MESA_FORMAT_RGBX_UNORM16] = BRW_SURFACEFORMAT_R16G16B16X16_UNORM,
+      [MESA_FORMAT_RGBX_SNORM16] = 0,
+      [MESA_FORMAT_RGBX_FLOAT16] = BRW_SURFACEFORMAT_R16G16B16X16_FLOAT,
+      [MESA_FORMAT_RGBX_UINT16] = 0,
+      [MESA_FORMAT_RGBX_SINT16] = 0,
+      [MESA_FORMAT_RGBX_FLOAT32] = BRW_SURFACEFORMAT_R32G32B32X32_FLOAT,
+      [MESA_FORMAT_RGBX_UINT32] = 0,
+      [MESA_FORMAT_RGBX_SINT32] = 0,
    };
    assert(mesa_format < MESA_FORMAT_COUNT);
    return table[mesa_format];
@@ -530,12 +532,12 @@ brw_init_surface_formats(struct brw_context *brw)
 {
    struct gl_context *ctx = &brw->ctx;
    int gen;
-   gl_format format;
+   mesa_format format;
 
    memset(&ctx->TextureFormatSupported, 0, sizeof(ctx->TextureFormatSupported));
 
    gen = brw->gen * 10;
-   if (brw->is_g4x)
+   if (brw->is_g4x || brw->is_haswell)
       gen += 5;
 
    for (format = MESA_FORMAT_NONE + 1; format < MESA_FORMAT_COUNT; format++) {
@@ -581,6 +583,9 @@ brw_init_surface_formats(struct brw_context *brw)
 	  */
 	 render = BRW_SURFACEFORMAT_B8G8R8A8_UNORM;
 	 break;
+      case BRW_SURFACEFORMAT_R8G8B8X8_UNORM:
+         render = BRW_SURFACEFORMAT_R8G8B8A8_UNORM;
+         break;
       }
 
       rinfo = &surface_formats[render];
@@ -600,34 +605,31 @@ brw_init_surface_formats(struct brw_context *brw)
    /* We will check this table for FBO completeness, but the surface format
     * table above only covered color rendering.
     */
-   brw->format_supported_as_render_target[MESA_FORMAT_S8_Z24] = true;
-   brw->format_supported_as_render_target[MESA_FORMAT_X8_Z24] = true;
-   brw->format_supported_as_render_target[MESA_FORMAT_S8] = true;
-   brw->format_supported_as_render_target[MESA_FORMAT_Z16] = true;
-   brw->format_supported_as_render_target[MESA_FORMAT_Z32_FLOAT] = true;
-   brw->format_supported_as_render_target[MESA_FORMAT_Z32_FLOAT_X24S8] = true;
+   brw->format_supported_as_render_target[MESA_FORMAT_Z24_UNORM_S8_UINT] = true;
+   brw->format_supported_as_render_target[MESA_FORMAT_Z24_UNORM_X8_UINT] = true;
+   brw->format_supported_as_render_target[MESA_FORMAT_S_UINT8] = true;
+   brw->format_supported_as_render_target[MESA_FORMAT_Z_FLOAT32] = true;
+   brw->format_supported_as_render_target[MESA_FORMAT_Z32_FLOAT_S8X24_UINT] = true;
 
    /* We remap depth formats to a supported texturing format in
     * translate_tex_format().
     */
-   ctx->TextureFormatSupported[MESA_FORMAT_S8_Z24] = true;
-   ctx->TextureFormatSupported[MESA_FORMAT_X8_Z24] = true;
-   ctx->TextureFormatSupported[MESA_FORMAT_Z32_FLOAT] = true;
-   ctx->TextureFormatSupported[MESA_FORMAT_Z32_FLOAT_X24S8] = true;
+   ctx->TextureFormatSupported[MESA_FORMAT_Z24_UNORM_S8_UINT] = true;
+   ctx->TextureFormatSupported[MESA_FORMAT_Z24_UNORM_X8_UINT] = true;
+   ctx->TextureFormatSupported[MESA_FORMAT_Z_FLOAT32] = true;
+   ctx->TextureFormatSupported[MESA_FORMAT_Z32_FLOAT_S8X24_UINT] = true;
 
-   /* It appears that Z16 is slower than Z24 (on Intel Ivybridge and newer
-    * hardware at least), so there's no real reason to prefer it unless you're
-    * under memory (not memory bandwidth) pressure.  Our speculation is that
-    * this is due to either increased fragment shader execution from
-    * GL_LEQUAL/GL_EQUAL depth tests at the reduced precision, or due to
-    * increased depth stalls from a cacheline-based heuristic for detecting
-    * depth stalls.
+   /* Benchmarking shows that Z16 is slower than Z24, so there's no reason to
+    * use it unless you're under memory (not memory bandwidth) pressure.
     *
-    * However, desktop GL 3.0+ require that you get exactly 16 bits when
-    * asking for DEPTH_COMPONENT16, so we have to respect that.
+    * Apparently, the GPU's depth scoreboarding works on a 32-bit granularity,
+    * which corresponds to one pixel in the depth buffer for Z24 or Z32 formats.
+    * However, it corresponds to two pixels with Z16, which means both need to
+    * hit the early depth case in order for it to happen.
+    *
+    * Other speculation is that we may be hitting increased fragment shader
+    * execution from GL_LEQUAL/GL_EQUAL depth tests at reduced precision.
     */
-   if (_mesa_is_desktop_gl(ctx))
-      ctx->TextureFormatSupported[MESA_FORMAT_Z16] = true;
 
    /* On hardware that lacks support for ETC1, we map ETC1 to RGBX
     * during glCompressedTexImage2D(). See intel_mipmap_tree::wraps_etc1.
@@ -654,7 +656,7 @@ bool
 brw_render_target_supported(struct brw_context *brw,
 			    struct gl_renderbuffer *rb)
 {
-   gl_format format = rb->Format;
+   mesa_format format = rb->Format;
 
    /* Many integer formats are promoted to RGBA (like XRGB8888 is), which means
     * we would consider them renderable even though we don't have surface
@@ -685,8 +687,7 @@ brw_render_target_supported(struct brw_context *brw,
 
 GLuint
 translate_tex_format(struct brw_context *brw,
-                     gl_format mesa_format,
-		     GLenum depth_mode,
+                     mesa_format mesa_format,
 		     GLenum srgb_decode)
 {
    struct gl_context *ctx = &brw->ctx;
@@ -695,18 +696,18 @@ translate_tex_format(struct brw_context *brw,
 
    switch( mesa_format ) {
 
-   case MESA_FORMAT_Z16:
-      return BRW_SURFACEFORMAT_I16_UNORM;
+   case MESA_FORMAT_Z_UNORM16:
+      return BRW_SURFACEFORMAT_R16_UNORM;
 
-   case MESA_FORMAT_S8_Z24:
-   case MESA_FORMAT_X8_Z24:
-      return BRW_SURFACEFORMAT_I24X8_UNORM;
+   case MESA_FORMAT_Z24_UNORM_S8_UINT:
+   case MESA_FORMAT_Z24_UNORM_X8_UINT:
+      return BRW_SURFACEFORMAT_R24_UNORM_X8_TYPELESS;
 
-   case MESA_FORMAT_Z32_FLOAT:
-      return BRW_SURFACEFORMAT_I32_FLOAT;
+   case MESA_FORMAT_Z_FLOAT32:
+      return BRW_SURFACEFORMAT_R32_FLOAT;
 
-   case MESA_FORMAT_Z32_FLOAT_X24S8:
-      return BRW_SURFACEFORMAT_R32G32_FLOAT;
+   case MESA_FORMAT_Z32_FLOAT_S8X24_UINT:
+      return BRW_SURFACEFORMAT_R32_FLOAT_X8X24_TYPELESS;
 
    case MESA_FORMAT_RGBA_FLOAT32:
       /* The value of this BRW_SURFACEFORMAT is 0, which tricks the
@@ -731,19 +732,60 @@ translate_tex_format(struct brw_context *brw,
    }
 }
 
+/**
+ * Convert a MESA_FORMAT to the corresponding BRW_DEPTHFORMAT enum.
+ */
+uint32_t
+brw_depth_format(struct brw_context *brw, mesa_format format)
+{
+   switch (format) {
+   case MESA_FORMAT_Z_UNORM16:
+      return BRW_DEPTHFORMAT_D16_UNORM;
+   case MESA_FORMAT_Z_FLOAT32:
+      return BRW_DEPTHFORMAT_D32_FLOAT;
+   case MESA_FORMAT_Z24_UNORM_X8_UINT:
+      if (brw->gen >= 6) {
+         return BRW_DEPTHFORMAT_D24_UNORM_X8_UINT;
+      } else {
+         /* Use D24_UNORM_S8, not D24_UNORM_X8.
+          *
+          * D24_UNORM_X8 was not introduced until Gen5. (See the Ironlake PRM,
+          * Volume 2, Part 1, Section 8.4.6 "Depth/Stencil Buffer State", Bits
+          * 3DSTATE_DEPTH_BUFFER.Surface_Format).
+          *
+          * However, on Gen5, D24_UNORM_X8 may be used only if separate
+          * stencil is enabled, and we never enable it. From the Ironlake PRM,
+          * same section as above, 3DSTATE_DEPTH_BUFFER's
+          * "Separate Stencil Buffer Enable" bit:
+          *
+          * "If this field is disabled, the Surface Format of the depth
+          *  buffer cannot be D24_UNORM_X8_UINT."
+          */
+         return BRW_DEPTHFORMAT_D24_UNORM_S8_UINT;
+      }
+   case MESA_FORMAT_Z24_UNORM_S8_UINT:
+      return BRW_DEPTHFORMAT_D24_UNORM_S8_UINT;
+   case MESA_FORMAT_Z32_FLOAT_S8X24_UINT:
+      return BRW_DEPTHFORMAT_D32_FLOAT_S8X24_UINT;
+   default:
+      assert(!"Unexpected depth format.");
+      return BRW_DEPTHFORMAT_D32_FLOAT;
+   }
+}
+
 /** Can HiZ be enabled on a depthbuffer of the given format? */
 bool
-brw_is_hiz_depth_format(struct brw_context *brw, gl_format format)
+brw_is_hiz_depth_format(struct brw_context *brw, mesa_format format)
 {
    if (!brw->has_hiz)
       return false;
 
    switch (format) {
-   case MESA_FORMAT_Z32_FLOAT:
-   case MESA_FORMAT_Z32_FLOAT_X24S8:
-   case MESA_FORMAT_X8_Z24:
-   case MESA_FORMAT_S8_Z24:
-   case MESA_FORMAT_Z16:
+   case MESA_FORMAT_Z_FLOAT32:
+   case MESA_FORMAT_Z32_FLOAT_S8X24_UINT:
+   case MESA_FORMAT_Z24_UNORM_X8_UINT:
+   case MESA_FORMAT_Z24_UNORM_S8_UINT:
+   case MESA_FORMAT_Z_UNORM16:
       return true;
    default:
       return false;
